@@ -1,18 +1,17 @@
 # Calc command.
 
+from distutils.version import LooseVersion
 import logging
-import sys
-import traceback
 
 import click
 import snuggs
 from cligj import files_inout_arg
 
+from .helpers import resolve_inout
+from . import options
 import rasterio
 from rasterio.fill import fillnodata
 from rasterio.features import sieve
-from rasterio.rio.cli import (
-    cli, dtype_opt, masked_opt, output_opt, resolve_inout)
 
 
 def get_bands(inputs, d, i=None):
@@ -31,16 +30,16 @@ def read_array(ix, subix=None, dtype=None):
     return arr
 
 
-@cli.command(short_help="Raster data calculator.")
+@click.command(short_help="Raster data calculator.")
 @click.argument('command')
 @files_inout_arg
-@output_opt
+@options.output_opt
 @click.option('--name', multiple=True,
               help='Specify an input file with a unique short (alphas only) '
                    'name for use in commands like '
                    '"a=tests/data/RGB.byte.tif".')
-@dtype_opt
-@masked_opt
+@options.dtype_opt
+@options.masked_opt
 @click.pass_context
 def calc(ctx, command, files, output, name, dtype, masked):
     """A raster data calculator
@@ -122,6 +121,10 @@ def calc(ctx, command, files, output, name, dtype, masked):
 
             res = snuggs.eval(command, **ctxkwds)
 
+            if (isinstance(res, np.ma.core.MaskedArray) and
+                    tuple(LooseVersion(np.__version__).version) < (1, 9, 0)):
+                res = res.filled(kwargs['nodata'])
+
             if len(res.shape) == 3:
                 results = np.ndarray.astype(res, dtype, copy=False)
             else:
@@ -139,8 +142,3 @@ def calc(ctx, command, files, output, name, dtype, masked):
         click.echo(' ' +  ' ' * err.offset + "^")
         click.echo(err)
         raise click.Abort()
-    #except Exception as err:
-    #    t, v, tb = sys.exc_info()
-    #    for line in traceback.format_exception_only(t, v):
-    #        click.echo(line, nl=False)
-    #    raise click.Abort()
